package bdd.search.spider;

import java.util.Enumeration;
import java.util.Hashtable;

/** Written by Tim Macinta 1997                           <br>
 *  Distributed under the GNU Public License
 *       (a copy of which is enclosed with the source).   <br>
 *                                                        <br> 
 * A WordExtractor should be able to extract the words from
 * a given file.  This class should be subclassed by classes which
 * understand different document types.
 */

public class WordExtractor {

  Hashtable words = new Hashtable(200);      // stores all the words
  int current_index = 0;        // index in document of current word

  public WordExtractor() {}

  /** Returns an Enumeration that returns each word in the document in
   *  no particular order.  A word is returned once at most regardless of
   *  the number of times it appears in the document.  The Enumeration
   *  returns a String for each call to nextElement().
   */
  public Enumeration getWords() {
    return words.keys();
  }

  /** Returns the number of words in this document.  */
  public int countWords() {
    return current_index;
  }
  
  /** Returns a count of the number of times that "word" appears in the
   *  the document.           */
  public int countOccurances(String word) {
    int[] stats = (int[]) words.get(word);
    if (stats == null) return 0;
    return stats[0];
  }

  /** Returns the index of "word".  The index is determined by counting
   *  the words in the document until the first occurance of "word" is
   *  found.  For instance, firstOccurance("the") would return 5 if the
   *  document started like this "Once upon a time the giant tomato of...".
   *  Returns -1 if the word is not in the document.
   */
  public int firstOccurance(String word) {
    int[] stats = (int[]) words.get(word);
    if (stats == null) return -1;
    return stats[1];
  }
  
  /** Used internally to add a word to the list of words as they are found
   *  in the document.  */
  public void addWord(String word) {
    word = word.toLowerCase();
    int[] stats = (int[]) words.get(word);
    current_index++;
    if (stats == null) {
      stats = new int[2];
      stats[0] = 0;               // number of time word appears
      stats[1] = current_index;   // first occurance of word
      words.put(word, stats);
    }
    stats[0]++;
  }

}
